@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2018
+    written by Marc Ihm 1995-2019
     more info at www.yabasic.de
 
     function.c --- code for functions
@@ -37,8 +37,8 @@ static double peek (char *);	/* peek into internals */
 static char *peek2 (char *, struct command *);	/* peek into internals */
 static char *peek3 (char *, char *);	/* peek into internals */
 static int peekfile (int);	/* read a byte from stream */
-static char *do_system (char *);	/* executes command via command.com */
-static int do_system2 (char *);	/* execute command as system */
+static int do_system (char *);	/* hand over execution of command to system, return exit code */
+static char *do_system2 (char *); /* hand over execution of command to system, return output as string */
 static double myrand (); /* generate random number in given range */
 
 /* ------------- global variables ---------------- */
@@ -463,6 +463,12 @@ function (struct command *current)	/* performs a function */
         }
         pointer = my_strdup (string);
         break;
+    case fCHOMP:
+        result = stSTRING;
+	pointer = a1->pointer;
+	a1->pointer = NULL;
+	pointer[strcspn(pointer, "\r\n")] = 0;
+	break;
     case fSQRT:
         value = sqrt (a1->value);
         result = stNUMBER;
@@ -774,13 +780,13 @@ function (struct command *current)	/* performs a function */
         break;
     case fSYSTEM:
         str = a1->pointer;
-        pointer = do_system (str);
-        result = stSTRING;
+        value = do_system (str);
+        result = stNUMBER;
         break;
     case fSYSTEM2:
         str = a1->pointer;
-        value = do_system2 (str);
-        result = stNUMBER;
+        pointer = do_system2 (str);
+        result = stSTRING;
         break;
     case fPEEK:
         str = a1->pointer;
@@ -845,7 +851,7 @@ function (struct command *current)	/* performs a function */
 
 
 static int
-do_system2 (char *cmd)		/* hand over execution of command to system */
+do_system (char *cmd)		/* hand over execution of command to system, return exit code */
 {
 #ifdef UNIX
     int ret;
@@ -854,11 +860,14 @@ do_system2 (char *cmd)		/* hand over execution of command to system */
     }
     ret = system (cmd);
     if (curinized) {
-        reset_prog_mode ();
+        if (tcsetpgrp(STDIN_FILENO, getpid())) {
+	    sprintf(string,"could not get control of terminal: %s",
+		    my_strerror(errno));
+	    error (ERROR,string);
+	    return ret;
+	}
+	reset_prog_mode ();
     }
-    signal(SIGTTOU, SIG_IGN);
-    signal(SIGTTIN, SIG_IGN);
-    tcsetpgrp(STDIN_FILENO, getpid());
 
     return ret;
 #else
@@ -956,7 +965,7 @@ recall_buff ()			/* recall store buffer */
 
 
 static char *
-do_system (char *cmd)		/* executes command via command.com */
+do_system2 (char *cmd)		/* hand over execution of command to system, return output as string */
 {
     static char buff[SYSBUFFLEN + 1];	/* buffer to store command */
     int len;			/* number of bytes read */
@@ -996,7 +1005,6 @@ do_system (char *cmd)		/* executes command via command.com */
         store_buff (buff, len);
     } while (c != EOF);
     pclose (p);
-
 #else
     ZeroMemory (&prosec, sizeof (prosec));
     prosec.nLength = sizeof (prosec);
@@ -1380,46 +1388,46 @@ void
 poke (struct command *cmd)	/* poke into internals */
 {
     char *dest, *s, c;
-    char *sarg = NULL;
-    double darg;
+    char *string_arg = NULL;
+    double double_arg;
     struct stackentry *stack;
     int count;
 
     if (cmd->tag == 's') {
-        sarg = pop (stSTRING)->pointer;
+        string_arg = pop (stSTRING)->pointer;
     } else {
-        darg = pop (stNUMBER)->value;
+        double_arg = pop (stNUMBER)->value;
     }
 
     dest = pop (stSTRING)->pointer;
     for (s = dest; *s; s++) {
         *s = tolower ((int) *s);
     }
-    if (!strcmp (dest, "fontheight") && !sarg) {
-        fontheight = (int) darg;
+    if (!strcmp (dest, "fontheight") && !string_arg) {
+        fontheight = (int) double_arg;
 #ifdef UNIX
         calc_psscale ();
 #endif
-    } else if (!strcmp (dest, "font") && sarg) {
-        fontname = my_strdup (sarg);
-    } else if (!strcmp (dest, "dump") && sarg) {
-        dump_commands (sarg);
-    } else if (!strcmp (dest, "dump") && sarg && !strcmp (sarg, "symbols")) {
+    } else if (!strcmp (dest, "font") && string_arg) {
+        fontname = my_strdup (string_arg);
+    } else if (!strcmp (dest, "dump") && string_arg) {
+        dump_commands (string_arg);
+    } else if (!strcmp (dest, "dump") && string_arg && !strcmp (string_arg, "symbols")) {
         dump_sym ();
-    } else if (!strcmp (dest, "dump") && sarg &&
-               (!strcmp (sarg, "sub") || !strcmp (sarg, "subs")
-                || !strcmp (sarg, "subroutine")
-                || !strcmp (sarg, "subroutines"))) {
+    } else if (!strcmp (dest, "dump") && string_arg &&
+               (!strcmp (string_arg, "sub") || !strcmp (string_arg, "subs")
+                || !strcmp (string_arg, "subroutine")
+                || !strcmp (string_arg, "subroutines"))) {
         dump_sub (0);
-    } else if (!strcmp (dest, "textalign") && sarg) {
-        if (!check_alignment (sarg)) {
+    } else if (!strcmp (dest, "textalign") && string_arg) {
+        if (!check_alignment (string_arg)) {
             return;
         }
-        strncpy (text_align, sarg, 2);
-    } else if (!strcmp (dest, "windoworigin") && sarg) {
-        moveorigin (sarg);
-    } else if (!strcmp (dest, "infolevel") && sarg) {
-        c = tolower ((int) *sarg);
+        strncpy (text_align, string_arg, 2);
+    } else if (!strcmp (dest, "windoworigin") && string_arg) {
+        moveorigin (string_arg);
+    } else if (!strcmp (dest, "infolevel") && string_arg) {
+        c = tolower ((int) *string_arg);
         switch (c) {
         case 'd':
             infolevel = DEBUG;
@@ -1444,19 +1452,19 @@ poke (struct command *cmd)	/* poke into internals */
             sprintf (string, "switching infolevel to '%c'", c);
             error (DEBUG, string);
         }
-    } else if (!strcmp (dest, "stdout") && sarg) {
-        fputs (sarg, stdout);
-    } else if (!strcmp (dest, "random_seed") && !sarg) {
-        srand((unsigned int) darg);
-    } else if (!strcmp (dest, "__assert_stack_size") && !sarg) {
+    } else if (!strcmp (dest, "stdout") && string_arg) {
+        fputs (string_arg, stdout);
+    } else if (!strcmp (dest, "random_seed") && !string_arg) {
+        srand((unsigned int) double_arg);
+    } else if (!strcmp (dest, "__assert_stack_size") && !string_arg) {
 	count = -1;
 	stack = stackhead;
 	while ( stack != stackroot) {
 	    count++;
 	    stack = stack->prev;
 	}
-	if (count != (int) darg) {
-	    sprintf (string, "assertion failed for number of entries on stack; expected = %d, actual = %d",(int) darg,count);
+	if (count != (int) double_arg) {
+	    sprintf (string, "assertion failed for number of entries on stack; expected = %d, actual = %d",(int) double_arg,count);
 	    error (FATAL, string);
 	}
     } else if (dest[0] == '#') {
@@ -1593,8 +1601,10 @@ peek2 (char *dest, struct command *curr)	/* peek into internals */
         return my_strdup(main_file_name);
     } else if (!strcmp (dest, "program_name")) {
         return my_strdup(progname);
+    } else if (!strcmp (dest, "interpreter_path")) {
+	return my_strdup(inter_path);
     } else if (!strcmp (dest, "library")) {
-        return my_strdup (curr->lib->s);
+        return my_strdup (curr->lib->short_name);
     } else if (!strcmp (dest, "version")) {
         return my_strdup (PACKAGE_VERSION);
     } else if (!strcmp (dest, "os")) {
