@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2018
+    written by Marc Ihm 1995-2019
     more info at www.yabasic.de
 
     io.c --- code for screen and file i/o
@@ -274,7 +274,16 @@ curinit(void)			/* initialize curses */
 #endif
 
 #ifdef UNIX
-    initscr();
+    if (tcsetpgrp(STDIN_FILENO, getpid())) {
+	sprintf(string,"could not get control of terminal: %s",
+                my_strerror(errno));
+        error (ERROR,string);
+	return;
+    };
+    if (!initscr()) {
+        error (ERROR,"could not initialize curses");
+	return;
+    };
     initcol();
     setscrreg(0,LINES);
     scrollok(stdscr,TRUE);
@@ -284,9 +293,6 @@ curinit(void)			/* initialize curses */
     set_escdelay(10);
     curs_set (0);
     intrflush(stdscr,FALSE);
-    signal(SIGTTOU, SIG_IGN);
-    signal(SIGTTIN, SIG_IGN);
-    tcsetpgrp(STDIN_FILENO, getpid());
 #else
     GetConsoleScreenBufferInfo(ConsoleOutput, &coninfo);
     COLS = coninfo.srWindow.Right + 1;
@@ -822,7 +828,7 @@ open_done:
 
 
 void
-checkopen(void)		/* check, if open has been sucessfull */
+checkopen(void)		/* check, if open has been sucessfully */
 {
     double result;
 
