@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2018
+    written by Marc Ihm 1995-2019
     more info at www.yabasic.de
 
     main.c --- main() and auxilliary functions
@@ -17,6 +17,8 @@
 #ifndef YABASIC_INCLUDED
 #include "yabasic.h"		/* all prototypes and structures */
 #endif
+#include "whereami.h"
+#include "bison.h"
 
 /* ------------- defines ---------------- */
 
@@ -24,7 +26,7 @@
 #define COPYRIGHT "Copyright 1995-2018 by Marc Ihm, according to the MIT License"
 /* NOTE: Including whatever timestamp into the banner would break the reproducible build */
 #define BANNER \
-"\nThis is yabasic version " VERSION ",\ncompiled on "\
+"\nThis is yabasic version " VERSION ",\nconfigured for "\
 ARCHITECTURE " at " BUILD_TIME "\n\n   " COPYRIGHT "\n\n"
 #define BANNER_VERSION \
 "yabasic " VERSION ", built on " ARCHITECTURE
@@ -33,7 +35,8 @@ ARCHITECTURE " at " BUILD_TIME "\n\n   " COPYRIGHT "\n\n"
 
 /* ------------- external references ---------------- */
 
-extern int mylineno;		/* current line number */
+extern int yylineno;            /* line number */
+extern YYLTYPE yylloc;          /* line numbers and columns */
 extern int yyparse ();		/* call bison parser */
 extern int yydebug;             /* for bison debugging */
 
@@ -51,8 +54,8 @@ int equal (char *, char *, int);	/* helper for processing options */
 static int mybind (char *);	/* bind a program to the interpreter and save it */
 void put_and_count (char *, FILE *, int *);  /* write text to file and increment len */
 
-char *find_interpreter (char *);	/* find interpreter with full path */
-static int seekback (FILE *, int);       /* seek back bytes */
+char *find_interpreter (char *);    /* find interpreter with full path, use code from Gregory Pakosz */
+static int seekback (FILE *, int, int);       /* seek back bytes */
 
 /* ------------- global variables ---------------- */
 
@@ -247,7 +250,7 @@ main (int argc, char **argv)
     }
 
     add_command (cEND, NULL, NULL);
-    sprintf (string, "read %d line(s) and generated %d command(s)", mylineno,
+    sprintf (string, "read %d line(s) and generated %d command(s)", yylineno,
              commandcount);
     error (NOTE, string);
 
@@ -271,7 +274,7 @@ main (int argc, char **argv)
         program_state = RUNNING;
 	if (infolevel >= DEBUG) {
 	    printf ("---Program parsed, press RETURN to continue with its execution: ");
-	    fgets (string, 2, stdin);
+	    fgets (string, INBUFFLEN, stdin);
         }
         run_it ();
     } else {
@@ -407,7 +410,9 @@ add_command (int type, char *symname, char *diag)
         std_diag ("creating", type, symname, diag);
     }
     cmdhead->type = type;		/* store command */
-    cmdhead->line = mylineno;
+    cmdhead->line = yylineno - currlib->yylineno_at_start + 1;
+    cmdhead->first_column = yylloc.first_column;
+    cmdhead->last_column = yylloc.last_column;
     cmdhead->lib = currlib;
     cmdhead->cnt = commandcount;
     if (!symname || !*symname) {
@@ -496,7 +501,7 @@ dump_commands (char *dumpfilename)
 
     for (cmd=cmdroot; cmd; cmd=cmd->next) {
         sprintf(string,"");
-        fprintf(dump,"Line %4d, Address %p:   %-*s   (lib %s, ptr %p, tag 0x%x%s)\n",cmd->line,cmd,max_explanation_length,explanation[cmd->type],cmd->lib->s,cmd->pointer,cmd->tag,string);
+        fprintf(dump,"Line %4d, Address %p:   %-*s   (lib %s, ptr %p, tag 0x%x%s)\n",cmd->line,cmd,max_explanation_length,explanation[cmd->type],cmd->lib->short_name,cmd->pointer,cmd->tag,string);
         if (cmd->type==cEND) {
             break;
         }
@@ -580,41 +585,43 @@ parse_arguments (int cargc, char *cargv[])
 		     "ARGUMENTS : strings, that are available from within the yabasic program\n\n");
 	    fprintf (stderr, "Available OPTIONS:\n");
 	    fprintf (stderr,
-		     "   --help                 : print this message\n");
+		     "   --help                  : print this message\n");
 	    fprintf (stderr,
-		     "   --version              : show version of yabasic\n");
+		     "   --version               : show version of yabasic\n");
 	    fprintf (stderr,
-		     "   -i,-infolevel [dnwefb] : set infolevel to debug,note,warning,error,fatal or bison\n");
+		     "   -i,--infolevel [dnwefb] : set infolevel to debug,note,warning,error,fatal or bison\n");
 	    fprintf (stderr,
-		     "   -e,--execute COMMANDS  : execute yabasic COMMANDS right away\n");
+		     "   -e,--execute COMMANDS   : execute yabasic COMMANDS right away\n");
 	    fprintf (stderr,
-		     "   --bind BOUND           : bind interpreter with FILENAME into BOUND\n");
+		     "   --bind BOUND            : bind interpreter with FILENAME into BOUND\n");
 	    fprintf (stderr,
-		     "   --geometry x+y         : position graphic window at x,y\n");
+		     "   --geometry x+y          : position graphic window at x,y\n");
 #ifdef UNIX
 	    fprintf (stderr,
-		     "   --fg,--bg COL          : specify fore/background color of graphic window\n");
+		     "   -fg,--foreground COL    : specify foreground color of graphic window\n");
 	    fprintf (stderr,
-		     "   --display DISP         : display, where window will show up\n");
+		     "   -bg,--background COL    : specify background color of graphic window\n");
 	    fprintf (stderr,
-		     "   --font FONT            : font for graphic window\n");
+		     "   --display DISP          : display, where window will show up\n");
+	    fprintf (stderr,
+		     "   --font FONT             : font for graphic window\n");
 #else
 	    fprintf (stderr,
-		     "   --font FONT            : font for graphic, supply style (decorative,dontcare,\n");
+		     "   --font FONT             : font for graphic, supply style (decorative,dontcare,\n");
 	    fprintf (stderr,
-		     "                            modern,roman,script or swiss) and size, e.g. swiss10\n");
+		     "                             modern,roman,script or swiss) and size, e.g. swiss10\n");
 #endif
 	    fprintf (stderr,
-		     "   --docu NAME            : print embedded docu of program or library\n");
+		     "   --docu NAME             : print embedded docu of program or library\n");
 	    fprintf (stderr,
-		     "   --check                : check for possible compatibility problems\n");
+		     "   --check                 : check for possible compatibility problems\n");
 	    fprintf (stderr,
-		     "   --                     : pass any subsequent words as arguments to yabasic\n");
+		     "   --librarypath PATH      : directory to search libraries not found in\n");
 	    fprintf (stderr,
-		     "   --librarypath PATH     : directory to search libraries not found in\n");
-	    fprintf (stderr,
-		     "                            current dir (default %s)\n",
+		     "                             current dir (default %s)\n",
 		     library_default);
+	    fprintf (stderr,
+		     "   --                      : pass any subsequent words as arguments to yabasic\n");
 	    fprintf (stderr, "\n");
 	    end_it ();
 	  } else if (equal ("-check", option, 2)) {
@@ -891,7 +898,7 @@ chop_command (char *command, int *argc, char ***argv)
 static void
 end_it (void)			/* perform shutdown-operations */
 {
-    char l[2];
+    char l[INBUFFLEN];
 #ifdef UNIX
     int status;
 
@@ -913,7 +920,7 @@ end_it (void)			/* perform shutdown-operations */
         SetConsoleMode (ConsoleInput, InitialConsole & (~ENABLE_ECHO_INPUT));
         FlushConsoleInputBuffer (ConsoleInput);
 #endif
-        fgets (l, 2, stdin);
+        fgets (l, INBUFFLEN, stdin);
 #ifdef WINDOWS
         if (wthandle != INVALID_HANDLE_VALUE) {
             TerminateThread (wthandle, 0);
@@ -959,13 +966,6 @@ initialize (void)
 #endif
     int i;
 
-#ifdef UNIX
-#ifdef SETPGRP_VOID
-    setpgrp ();
-#else
-#endif
-#endif
-
     /* install exception handler */
     signal (SIGFPE, signal_handler);
     signal (SIGSEGV, signal_handler);
@@ -1327,7 +1327,7 @@ run_it ()
                 l++;
                 if (hold_docu && !(l % 24)) {
                     printf ("---Press RETURN to continue ");
-                    fgets (string, 2, stdin);
+                    fgets (string, INBUFFLEN, stdin);
                 }
             } else {
                 if (infolevel >= DEBUG) {
@@ -1341,7 +1341,7 @@ run_it ()
         }
         if (hold_docu) {
             printf ("---End of embbedded documentation, press RETURN ");
-            fgets (string, 2, stdin);
+            fgets (string, INBUFFLEN, stdin);
         }
     } else {
         while (current != cmdhead && endreason == erNONE) {
@@ -1716,26 +1716,24 @@ run_it ()
 
 
 void
-error (int severity, char *messageline)
-/* reports an basic error to the user and possibly exits */
+error (int severity, char *message)
+/* reports an error to the user and possibly exits */
 {
     if (program_state == COMPILING) {
-        error_with_line (severity, messageline, mylineno );
+        error_with_position (severity, message, currlib->long_name, yylineno - currlib->yylineno_at_start + 1, yylloc.first_column, yylloc.last_column );
     } else if (program_state == RUNNING && current->line > 0) {
-        error_with_line (severity, messageline, current->line);
+        error_with_position (severity, message, current->lib->long_name, current->line, current->first_column, current->last_column);
     } else {
-        error_with_line (severity, messageline, -1);
+        error_without_position (severity, message);
     }
 }
 
 
 void
-error_with_line (int severity, char *message, int line)
+error_with_position (int severity, char *message, char *filename, int lineno, int first_column, int last_column)
 /* reports an basic error to the user and possibly exits */
 {
-    char *stext;
-    char *f = NULL;
-    int l;
+    char *severity_text;
     static int lastline;
     static int first = TRUE;
 
@@ -1748,51 +1746,45 @@ error_with_line (int severity, char *message, int line)
 
         switch (severity) {
         case (INFO):
-            stext = "---Info";
+            severity_text = "---Info";
             break;
         case (DUMP):
-            stext = "---Dump";
+            severity_text = "---Dump";
             break;
         case (DEBUG):
-            stext = "---Debug";
+            severity_text = "---Debug";
             debug_count++;
             break;
         case (NOTE):
-            stext = "---Note";
+            severity_text = "---Note";
             note_count++;
             break;
         case (WARNING):
-            stext = "---Warning";
+            severity_text = "---Warning";
             warning_count++;
             break;
         case (ERROR):
-            stext = "---Error";
+            severity_text = "---Error";
             error_count++;
             break;
         case (FATAL):
-            stext = "---Fatal";
+            severity_text = "---Fatal";
             break;
         }
-        fprintf (stderr, "%s", stext);
-        if (line >= 0) {
-            if (program_state == COMPILING) {
-                f = currlib->l;
-                l = line;
-            } else if (program_state == RUNNING && current->line > 0) {
-                f = current->lib->l;
-                l = current->line;
-            }
-            if (f) {
-                if (first || lastline != l) {
-                    fprintf (stderr, " in %s, line %d", f, l);
-                }
-                lastline = l;
-                first = FALSE;
-            }
-        }
-        fprintf (stderr, ": %s\n", message);
-        if (program_state == RUNNING && severity <= ERROR
-                && severity != DUMP) {
+        fprintf (stderr, "%s", severity_text);
+	if (filename) {
+	    if (first || lastline != lineno) {
+		fprintf (stderr, " in %s, line %d: %s\n", filename, lineno, message);
+		if (severity==ERROR) {
+		   show_and_mark_line (filename, lineno, first_column, last_column);
+		}
+	    }
+	    lastline = lineno;
+	    first = FALSE;
+        } else {
+	    fprintf (stderr, ": %s\n", message);
+	}
+        if (program_state == RUNNING && severity <= ERROR && severity != DUMP) {
             dump_sub (1);
         }
     }
@@ -1818,8 +1810,51 @@ error_with_line (int severity, char *message, int line)
 }
 
 
+void
+show_and_mark_line (char *filename, int lineno, int first_column, int last_column) /* try to find and show offending line */
+{
+    FILE *file;
+    char linebuffer[INBUFFLEN];
+    int i;
+    
+    if (is_bound) {
+	return;
+    }
+
+    file = fopen(filename, "r");
+    if (!file) {
+	return;
+    }
+
+    linebuffer[0]='\0';
+    while (lineno && fgets(linebuffer, INBUFFLEN, file)) {
+	lineno--;
+    }
+    fclose(file);
+
+    if (linebuffer[0] && lineno==0) {
+	linebuffer[strcspn(linebuffer, "\n")] = '\0';
+	fputs("   ", stderr);
+	fputs(linebuffer, stderr);
+	fputs("\n   ", stderr);
+	for(i=1;i<=last_column;i++) {
+	    fputc((i<first_column) ? ' ': ((i==first_column) ? '^':'~'), stderr);
+	}
+	fputs("\n", stderr);
+    }
+}
+
+
+void
+error_without_position(int severity, char *message)
+/* like error_with_position but without position */
+{
+    error_with_position (severity, message, NULL, 0, 0, 0);
+}
+
+
 char *
-my_strndup (char *arg, int len)	/*  own version of strndup */
+my_strndup (char *arg, int len)	/* own version of strndup */
 {
     char *copy;
 
@@ -1866,17 +1901,17 @@ my_free (void *mem)		/* free memory */
 }
 
 
-struct libfile_name *
-new_file (char *l, char *s)  	/* create a new structure for library names */
+struct library *
+new_library (char *long_name, char *short_name)  	/* create a new structure for library */
 {
-    struct libfile_name *new;
-    struct libfile_name *curr;
-    static struct libfile_name *last = NULL;
+    struct library *new;
+    struct library *curr;
+    static struct library *last = NULL;
     int start, end;
 
-    /* check, if library has already been included */
-    for (curr = libfile_stack[0]; curr; curr = curr->next) {
-        if (!strcmp (curr->l, l)) {
+    /* check, if library has already been imported */
+    for (curr = library_chain[0]; curr; curr = curr->next_lib) {
+        if (!strcmp (curr->long_name, long_name) && currlib->imported_from != curr->imported_from) {
             if (is_bound) {
                 return curr;
             } else {
@@ -1885,36 +1920,36 @@ new_file (char *l, char *s)  	/* create a new structure for library names */
         }
     }
 
-    new = my_malloc (sizeof (struct libfile_name));
-    new->next = NULL;
-    new->lineno = 1;
+    new = my_malloc (sizeof (struct library));
+    new->next_lib = NULL;
     if (last) {
-        last->next = new;
+        last->next_lib = new;
     }
     last = new;
 
-    new->l = my_strdup (l);
-    new->llen = strlen (new->l);
+    new->long_name = my_strdup (long_name);
+    new->long_len = strlen (new->long_name);
+    new->yylineno_at_start = yylineno;
 
-    if (s) {
-        new->s = my_strdup (s);
+    if (short_name) {
+        new->short_name = my_strdup (short_name);
     } else {
-        /* no short name supplied; get piece from l */
-        end = strlen (l);
+        /* no short name supplied; get piece from long_name */
+        end = strlen (long_name);
         for (start = end; start > 0; start--) {
-            if (l[start - 1] == '\\' || l[start - 1] == '/') {
+            if (long_name[start - 1] == '\\' || long_name[start - 1] == '/') {
                 break;
             }
-            if (l[start] == '.') {
+            if (long_name[start] == '.') {
                 end = start;
             }
         }
         end--;
-        new->s = my_malloc (end - start + 2);
-        strncpy (new->s, new->l + start, end - start + 1);
-        new->s[end - start + 1] = '\0';
+        new->short_name = my_malloc (end - start + 2);
+        strncpy (new->short_name, new->long_name + start, end - start + 1);
+        new->short_name[end - start + 1] = '\0';
     }
-    new->slen = strlen (new->s);
+    new->short_len = strlen (new->short_name);
     new->datapointer = new->firstdata = NULL;
 
     return new;
@@ -1926,7 +1961,7 @@ dotify (char *name, int addfun)	/* add library name, if not already present */
 {
     static char buff[200];
     if (!strchr (name, '.')) {
-	strncpy (buff, currlib->s, 200);
+	strncpy (buff, currlib->short_name, 200);
         strncat (buff, ".", 200-1-strlen(buff));
         strncat (buff, name, 200-1-strlen(buff));
     } else {
@@ -2125,7 +2160,7 @@ isbound (void)			/* check if this interpreter is bound to a program */
 
     /* check magic cookie */
     offset -= 1 + strlen (YABMAGIC);
-    if (!seekback (inter, offset)) return 0;
+    if (!seekback (inter, offset, FALSE)) return 0;
     for (i = 0; i < (int) strlen (YABMAGIC); i++) {
         c = fgetc (inter);
         if (c == EOF || c != (YABMAGIC)[i]) {
@@ -2139,13 +2174,13 @@ isbound (void)			/* check if this interpreter is bound to a program */
 
     /* infolevel */
     offset -= remlen + 2;
-    if (!seekback (inter, offset)) return 0;
+    if (!seekback (inter, offset, TRUE)) return 0;
     if (!fscanf (inter, "%d", &infolevel)) {
         error (WARNING, "Could not read infolevel");
         return 0;
     }
     /* repeat just for its output side-effect */
-    if (!seekback (inter, offset)) return 0; 
+    if (!seekback (inter, offset, TRUE)) return 0; 
     switch(infolevel) {
     case FATAL: infolevel_text="FATAL";break;
     case ERROR: infolevel_text="ERROR";break;
@@ -2160,7 +2195,7 @@ isbound (void)			/* check if this interpreter is bound to a program */
 
     /* length of name of embedded program */
     offset -= remlen + 8;
-    if (!seekback (inter, offset)) return 0;
+    if (!seekback (inter, offset, TRUE)) return 0;
     if (!fscanf (inter, "%d", &namelen)) {
         error (WARNING, "Could not read length of name of embedded program");
         return 0;
@@ -2170,7 +2205,7 @@ isbound (void)			/* check if this interpreter is bound to a program */
     
     /* name of embedded program */
     offset -= remlen + namelen;
-    if (!seekback (inter, offset)) return 0;
+    if (!seekback (inter, offset, TRUE)) return 0;
     progname = (char *) my_malloc (sizeof (char) * (namelen + 1));
     if (!fgets (progname, namelen + 1, inter)) {
         error (WARNING, "Could not read name of embedded program");
@@ -2181,7 +2216,7 @@ isbound (void)			/* check if this interpreter is bound to a program */
 
     /* length of program */
     offset -= remlen + 8;
-    if (!seekback (inter, offset)) return 0;
+    if (!seekback (inter, offset, TRUE)) return 0;
     if (!fscanf (inter, "%d", &proglen)) {
         error (WARNING, "Could not read length of embedded program");
         return 0;
@@ -2191,7 +2226,7 @@ isbound (void)			/* check if this interpreter is bound to a program */
 
     /* seek back to start of embedded program */
     offset -= 4 + proglen; /* only the text 'rem ' without preceding linefeed */
-    if (!seekback (inter, offset)) return 0;
+    if (!seekback (inter, offset, TRUE)) return 0;
 
     if (infolevel >= NOTE) {
         error (NOTE, "Dumping the embedded program, that will be executed:");
@@ -2206,8 +2241,8 @@ isbound (void)			/* check if this interpreter is bound to a program */
 	fprintf (stderr, "\n");
         error (NOTE, "End of program, that will be executed");
 	printf ("---Press RETURN to continue with its parsing: ");
-	fgets (string, 2, stdin);
-	if (!seekback (inter, offset)) return 0;
+	fgets (string, INBUFFLEN, stdin);
+	if (!seekback (inter, offset, TRUE)) return 0;
     }
     bound_program = inter;
     return 1;
@@ -2215,7 +2250,7 @@ isbound (void)			/* check if this interpreter is bound to a program */
 
 
 static int
-seekback (FILE *file, int offset)           /* seek back bytes */
+seekback (FILE *file, int offset, int cookie_found)           /* seek back bytes */
 {
   if (fseek (file, offset, SEEK_END)) {
     sprintf (errorstring, "Couldn't seek within '%s': %s", inter_path,
@@ -2224,15 +2259,16 @@ seekback (FILE *file, int offset)           /* seek back bytes */
     return FALSE;
   }
   if (!fgets (string, INBUFFLEN, file)) {
-    error (WARNING, "Could not read from end of embedded program");
+    error (cookie_found ? WARNING:DEBUG, "Could not read from end of embedded program");
     return FALSE;
   }
   string[strlen(string) - strlen("\n")] = '\0';
-  sprintf(errorstring, "Next line from end of embbeded program to be processed is: '%s'", string);
-  error (DEBUG, errorstring);
+  if (cookie_found && infolevel >= DEBUG) { 
+    sprintf(errorstring, "Next line from end of embbeded program to be processed is: '%s'", string);
+    error (DEBUG, errorstring);
+  }
   if (fseek (file, offset, SEEK_END)) {
-    sprintf (errorstring, "Couldn't seek within '%s': %s", inter_path,
-	     my_strerror (errno));
+    sprintf (errorstring, "Couldn't seek within '%s': %s", inter_path, my_strerror (errno));
     error (WARNING, errorstring);
     return FALSE;
   }
@@ -2301,15 +2337,15 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
     while ((c = fgetc (fyab)) != EOF) {
         fputc (c, fbound);
     }
-    for (i = 1; i < libfile_chain_length; i++) {
-        if (!(flib = fopen (libfile_chain[i]->l, "rb"))) {
+    for (i = 1; i < library_chain_length; i++) {
+        if (!(flib = fopen (library_chain[i]->long_name, "rb"))) {
             sprintf (string, "could not open '%s' for reading: %s",
-                     libfile_chain[i]->l, my_strerror (errno));
+                     library_chain[i]->long_name, my_strerror (errno));
             error (ERROR, string);
             fclose (flib);
             return 0;
         }
-        sprintf (string, "\nimport %s\n", libfile_chain[i]->s);
+        sprintf (string, "\nimport %s\n", library_chain[i]->short_name);
 	put_and_count(string, fbound, &proglen);
 	
 	put_and_count("\nimport __IGNORE_NESTED_IMPORTS\n", fbound, &proglen);
@@ -2353,48 +2389,17 @@ put_and_count (char *text, FILE *file, int *len)  /* write text to file and incr
 
 
 char *
-find_interpreter (char *name)	/* find interpreter with full path */
+find_interpreter (char *name)	/* find interpreter with full path, use code from Gregory Pakosz */
 {
-    FILE *f;
     char *path = NULL;
+    int length, dirname_length;
+    
+    length = wai_getExecutablePath(NULL, 0, NULL);
+    path = (char*)malloc(length + 1);
+    wai_getExecutablePath(path, length, &dirname_length);
+    path[length] = '\0';
 
-#ifdef WINDOWS
-    path = my_malloc (1000);
-    GetModuleFileName (NULL, path, 1000);
-    if (f = fopen (path, "r")) {
-        fclose (f);
-        return path;
-    } else {
-        my_free (path);
-        return my_strdup (name);
-    }
-
-#else
-    if ((f = fopen (name, "r"))) {
-        fclose (f);
-        path = my_strdup (name);
-        return path;
-    } else {
-        char *from, *to, *try;
-        from = to = path = getenv ("PATH");
-        try = my_malloc (strlen (path) + strlen (name) + 2);
-        to = strchr (to + 1, ':');
-        while (to) {
-            strncpy (try, from, to - from);
-            try[to - from] = 0;
-            if (try[to - from - 1] != '/')
-                    strcat (try, "/");
-            strcat (try, name);
-            if ((f = fopen (try, "r"))) {
-                fclose (f);
-                return try;
-            }
-            from = to + 1;
-            to = strchr (to + 1, ':');
-        }
-        return name;
-    }
-#endif
+    return path;
 }
 
 
