@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2018
+    written by Marc Ihm 1995-2019
     more info at www.yabasic.de
 
     symbol.c --- code for symbol, stack and library management, handling of arrays
@@ -1233,7 +1233,7 @@ doarray (struct command *cmd)	/* call an array */
 
     ar = ((struct symbol *) cmd->symbol)->pointer;
 
-    if (!ar->dimension) {
+    if (!ar) {
         sprintf (string, "array parameter '%s()' has not been supplied",
                  strip (cmd->symname));
         error (ERROR, string);
